unit fMain;
{$INCLUDE CADSoftTools.INC}
//If you see error "File not found: 'CADSoftTools.INC'",
//then please contact info@cadsofttools.com to get a compilable version.
//If you have a compilable version, you need to copy the contents of
// Lib7/LibXE5win32/LibXE5win64... to Lib directory according to your
// Delphi version and target CPU platform.

{$INCLUDE SGDXF.INC}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtDlgs, Menus, ExtCtrls, sgDrawingNavigator, CADImage,
  sgSelection, DXFConv, sgConsts, Buttons, ComCtrls, StdCtrls,
  DXF, DWG, DWG12, HPGL2, SVG, CGM, DWF,
  ToolWin, sgSManager, sgBlockLibrary, Grids, XMLInterface, fDeveloperIDE;

type
  TfmMain = class(TForm)
    mmmain: TMainMenu;
    mmiFile: TMenuItem;
    mmiOpen: TMenuItem;
    mmiClose: TMenuItem;
    N1: TMenuItem;
    mmiExit: TMenuItem;
    OpenDialog: TOpenPictureDialog;
    mmiSelection: TMenuItem;
    sbrMain: TStatusBar;
    mmiSelectAll: TMenuItem;
    mmiDeselectAll: TMenuItem;
    pnlMain: TPanel;
    BtnOpen: TSpeedButton;
    btnClose: TSpeedButton;
    pnlSelection: TPanel;
    btnCopy: TSpeedButton;
    btnPaste: TSpeedButton;
    Preview1: TMenuItem;
    pnlButtons: TPanel;
    pnlEntInsp: TPanel;
    BtnSelect: TSpeedButton;
    BtnMSelect: TSpeedButton;
    btnEntityInspector: TSpeedButton;
    splImage: TSplitter;
    sbPreviewSelection: TSpeedButton;
    SaveDialog1: TSaveDialog;
    Save: TMenuItem;
    CheckBox1: TCheckBox;
    procedure FileOpen;
    procedure FormCreate(Sender: TObject);
    procedure sbxImageBoxResize(Sender: TObject);
    procedure Select;
    procedure MSelect;
    procedure FileClose;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure spbtnMSelectClick(Sender: TObject);
    procedure spbtnSelectClick(Sender: TObject);
    procedure mmiExitClick(Sender: TObject);
    procedure mmiOpenClick(Sender: TObject);
    procedure mmiCloseClick(Sender: TObject);
    procedure mmiSelectAllClick(Sender: TObject);
    procedure mmiDeselectAllClick(Sender: TObject);
    procedure BtnSelectClick(Sender: TObject);
    procedure BtnMSelectClick(Sender: TObject);
    procedure BtnOpenClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure btnPasteClick(Sender: TObject);
    procedure btnCopyClick(Sender: TObject);
    procedure Preview1Click(Sender: TObject);
    procedure btnEntityInspectorClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SaveClick(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
  private
    FManager: TsgSelectionManager;
    FDNavigator: TsgDrawingNavigator;
    procedure FMSChange(var Message: TMessage); message SM_SELECTIONCHANGED;
    procedure DNMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
  public
    FBlockLibrary: TsgBlockLibrary;
    procedure OnSelectionChange;
    procedure OnSetValue;
    procedure InsertBlock(AIndex: Integer; X, Y: Single);
    property DNavigator: TsgDrawingNavigator read FDNavigator;
  end;

var
  fmMain: TfmMain;

implementation

uses
  fEntityInspector, BlockInsert, fPreview
  {$IFDEF DEMO}
  , fInfo
  {$ENDIF}
  ;

{$R *.dfm}
//
//type
//  TsgCADImageAccess = class(TsgCADImage);

function sgFloatToStrF(Value: TsgFloat): string;
begin
 if Value < 10000000 then
   Result := FloatToStrF(Value, ffGeneral, 7, 0)
 else
   Result := Format('%.7e', [Value]);
end;

procedure TfmMain.FMSChange(var Message: TMessage);
begin
  OnSelectionChange;
end;

procedure TfmMain.DNMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  vPt: TFPoint;
begin
  sbrMain.Panels[1].Text := 'X:' + IntToStr(X) + '  Y:' + IntToStr(Y);
  vPt := DNavigator.GetDrawingInternalCoordsWithoutSnap(X, Y);
  sbrMain.Panels[2].Text := 'X:' + sgFloatToStrF(vPt.X) + '  Y:' + sgFloatToStrF(vPt.Y) +
    '  Z:' + sgFloatToStrF(vPt.Z);
end;

procedure TfmMain.FileOpen;
var
  I: Integer;
begin
  FManager.DeselectAll;
  OpenDialog.FileName := '';
  if not OpenDialog.Execute then
    Exit;
  DNavigator.Picture.Graphic := nil;
  DNavigator.BeginUpdate;
  try
    DNavigator.LoadFromFile(OpenDialog.FileName);
    FManager.DNavigator := DNavigator;
    FBlockLibrary.DNavigator := DNavigator;
    Caption := OpenDialog.FileName + ' Editor';
    BtnSelect.Enabled := True;
    BtnMSelect.Enabled := True;
    BtnClose.Enabled := True;
    BtnSelect.Down := True;
    BtnMSelect.Down := False;
    mmiClose.Enabled := True;
    mmiSelection.Enabled := True;
    DNavigator.Align := alClient;
    DNavigator.FitToSize;
    if DNavigator.Picture.Graphic is TsgCADImage then
    begin
      TsgCADImage(DNavigator.Picture.Graphic).SetMatrixMode(smEnabled);
      if TsgCADImage(DNavigator.Picture.Graphic).SelectionMatrix = nil then
        TsgCADImage(DNavigator.Picture.Graphic).SelectionMatrix := TsgSelectionMatrix.Create;
      fmEntityInspector.Clear;
      fmDeveloperIDE.CADImage := TsgCADImage(DNavigator.Picture.Graphic);
    end;
  finally
    DNavigator.EndUpdate;
    DNavigator.Update;
  end;
  sbrMain.Panels[0].Text := OpenDialog.FileName;
  for I := 1 to sbrMain.Panels.Count - 1 do
    sbrMain.Panels[I].Text := '';
end;

procedure TfmMain.FormCreate(Sender: TObject);
begin
  sSHXSearchPaths := ExpandFileName('../../../SHX');
  fmDeveloperIDE := TfmDeveloperIDE.Create(Self);
  FDNavigator := TsgDrawingNavigator.Create(Self);
  FDNavigator.Parent := Self;
  FDNavigator.AutoFocus := True;
  FDNavigator.RectZooming := True;
  FDNavigator.OnMouseMove := DNMouseMove;
  FManager := TsgSelectionManager.Create;
  FBlockLibrary := TsgBlocklibrary.Create;
  FManager.Mode := mmSelect;
  BtnSelect.Down := False;
  BtnSelect.Enabled := False;
  BtnMSelect.Down := False;
  BtnMSelect.Enabled := False;
  BtnClose.Enabled := False;
  mmiClose.Enabled := False;
  mmiSelection.Enabled := False;
  {$IFDEF SGDEL_4}
  pnlEntInsp.Anchors := [akLeft, akTop, akRight, akBottom];
  pnlEntInsp.BiDiMode := bdRightToLeft;
  pnlEntInsp.DockSite := True;
  pnlEntInsp.ParentBiDiMode := False;
  {$ENDIF}
  {$IFDEF DEMO}
  fInfo.CreateInfoFrame(FDNavigator.Handle, Point(1, 1));
  {$ENDIF}
end;

procedure TfmMain.sbxImageBoxResize(Sender: TObject);
begin
  if not DNavigator.Empty then
    DNavigator.SetBounds(DNavigator.Left,DNavigator.Top,DNavigator.Width,DNavigator.Height);
end;

procedure TfmMain.Select;
begin
  DNavigator.Cursor := crArrow;
  FManager.Mode := mmSelect;
end;

procedure TfmMain.MSelect;
begin
  DNavigator.Cursor := crCross;
  FManager.Mode := mmMultiSelect;
end;

procedure TfmMain.FileClose;
var
  I: Integer;
begin
  FManager.Cancel;
  DNavigator.Picture.Graphic := nil;
  DNavigator.Align := alNone;
  DNavigator.SetBounds(0,0,0,0);
  Caption := Application.Title;
  BtnSelect.Down := False;
  BtnSelect.Enabled := False;
  BtnMSelect.Down := False;
  BtnMSelect.Enabled := False;
  BtnClose.Enabled := False;
  mmiSelection.Enabled := False;
  for I := 0 to sbrMain.Panels.Count - 1 do
    sbrMain.Panels[I].Text := '';
end;

procedure TfmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FBlockLibrary.Free;
  FileClose;
  FManager.Free;
end;

procedure TfmMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    FManager.DeselectAll;
end;

procedure TfmMain.OnSelectionChange;
begin
  MessageBox(Self.Handle, 'Selection is changed!!!', 'Information', 0);
end;

procedure TfmMain.OnSetValue;
begin
  DNavigator.Invalidate;
end;

procedure TfmMain.spbtnMSelectClick(Sender: TObject);
begin
  MSelect;
end;

procedure TfmMain.spbtnSelectClick(Sender: TObject);
begin
  Select;
end;

procedure TfmMain.mmiExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfmMain.mmiOpenClick(Sender: TObject);
begin
  FileOpen;
end;

procedure TfmMain.mmiCloseClick(Sender: TObject);
begin
  FileClose;
end;

procedure TfmMain.mmiSelectAllClick(Sender: TObject);
begin
  FManager.SelectAll;
end;

procedure TfmMain.mmiDeselectAllClick(Sender: TObject);
begin
  FManager.DeselectAll;
end;

procedure TfmMain.BtnSelectClick(Sender: TObject);
begin
  Select;
  btnSelect.Down := True;
  btnMSelect.Down := False;
end;

procedure TfmMain.CheckBox1Click(Sender: TObject);
begin
 // CheckBox1.Checked := not CheckBox1.Checked;
  fmEntityInspector.IndexesAsPluses := CheckBox1.Checked;
end;

procedure TfmMain.BtnMSelectClick(Sender: TObject);
begin
  MSelect;
  btnSelect.Down := False;
  btnMSelect.Down := True;
end;

procedure TfmMain.BtnOpenClick(Sender: TObject);
begin
  FileOpen;
end;

procedure TfmMain.btnCloseClick(Sender: TObject);
begin
  FileClose;
end;

procedure TfmMain.btnPasteClick(Sender: TObject);
begin
  fmBlockInsert.ShowModal;
end;

procedure TfmMain.btnCopyClick(Sender: TObject);
begin
  if (FManager.Entities <> nil) and (FManager.Entities.Count > 0) then
    FBlockLibrary.CopyAsBlock(FManager.Entities);
end;

procedure TfmMain.Preview1Click(Sender: TObject);
begin
  CreatePreview(Self, FManager, TsgCADImage(DNavigator.Picture.Graphic));
end;

procedure TfmMain.SaveClick(Sender: TObject);
var
  vXML: TsgCADtoXML;
begin
  if (DNavigator.Picture.Graphic <> nil) and (DNavigator.Picture.Graphic is TsgCADImage) then
  begin
    if not SaveDialog1.Execute then
      Exit;
    vXML := TsgCADtoXML.Create;
    try
      vXML.ExportEntity(TsgCADImage(DNavigator.Picture.Graphic).Converter);
      vXML.XML.SaveToFile(SaveDialog1.FileName);
    finally
      vXML.Free;
    end;
  end;
end;

procedure TfmMain.btnEntityInspectorClick(Sender: TObject);
begin
 if fmEntityInspector.Visible then
   fmEntityInspector.Hide
 else
   fmEntityInspector.Show;
end;

procedure TfmMain.InsertBlock(AIndex: Integer; X, Y: Single);
begin
  fmEntityInspector.AddEntity(FBlockLibrary.PasteAsBlock(AIndex, X, Y), True);
end;

procedure TfmMain.FormResize(Sender: TObject);
var
  I, vSize: Integer;
begin
  vSize := 0;
  for I := 1 to sbrMain.Panels.Count - 1 do
    Inc(vSize, sbrMain.Panels[I].Width);
  sbrMain.Panels[0].Width := sbrMain.Width - vSize;
end;

procedure TfmMain.FormShow(Sender: TObject);
begin
  fmEntityInspector.Dock(pnlEntInsp, pnlEntInsp.ClientRect);
  fmEntityInspector.Show;
  fmEntityInspector.OnSetValue := OnSetValue;
  //uncomment to work with Developer IDE XML form
//  fmDeveloperIDE.show;
//  fmDeveloperIDE.BringToFront;
end;

end.
